@@ -41,8 +41,9 @@ def process_job(folder: Union[Path, str]) -> None:
     logger.info(f"Loading pickle: {paths.submitted_pickle}")
     wait_time = 60
     for _ in range(wait_time):
-        if not paths.submitted_pickle.exists():
-            time.sleep(1)
+        if paths.submitted_pickle.exists():
+            break
+        time.sleep(1)
     if not paths.submitted_pickle.exists():
         raise RuntimeError(
             f"Waited for {wait_time} seconds but could not find submitted jobs in path:\n{paths.submitted_pickle}"
@@ -52,9 +53,12 @@ def process_job(folder: Union[Path, str]) -> None:
         env = job_environment.JobEnvironment()
         env._handle_signals(paths, delayed)
         result = delayed.result()
+        logger.info("Job completed successfully")
+        del delayed  # if it blocks here, you have a race condition that must be solved!
         with utils.temporary_save_path(paths.result_pickle) as tmppath:  # save somewhere else, and move
             utils.cloudpickle_dump(("success", result), tmppath)
-            logger.info("Job completed successfully")
+            del result
+            logger.info("Exiting after successful completion")
     except Exception as error:  # TODO: check pickle methods for capturing traceback; pickling and raising
         try:
             with utils.temporary_save_path(paths.result_pickle) as tmppath:
