@@ -5,6 +5,7 @@
 #
 
 import collections
+import contextlib
 import datetime
 import itertools
 import os
@@ -175,7 +176,7 @@ class RsyncSnapshot:
     def __init__(
         self,
         snapshot_dir: Path,
-        root_dir: Path = None,
+        root_dir: tp.Optional[Path] = None,
         with_submodules: bool = False,
         exclude: tp.Sequence[str] = (),
         include: tp.Sequence[str] = (),
@@ -291,8 +292,36 @@ def monitor_jobs(
     print(f"Whole process is finished, took {int((time.time() - monitoring_start_time) / 60)} minutes")
 
 
+@contextlib.contextmanager
+def clean_env() -> tp.Iterator[None]:
+    """Removes slurm and submitit related environment variables so as to avoid interferences
+    when submiting a new job from a job.
+
+    Note
+    ----
+    A slurm job submitted from within a slurm job inherits some of its attributes, which may
+    be confusing a cause weird gres errors (or pytorch distributed).
+    Submitting within this context should prevent this.
+
+    Usage
+    -----
+    with submitit.helpers.clean_env():
+        executor.submit(...)
+    """
+    distrib_names = ("MASTER_ADDR", "MASTER_PORT", "RANK", "WORLD_SIZE", "LOCAL_RANK", "LOCAL_WORLD_SIZE")
+    cluster_env = {
+        x: os.environ.pop(x)
+        for x in os.environ
+        if x.startswith(("SLURM_", "SLURMD_", "SRUN_", "SBATCH_", "SUBMITIT_")) or x in distrib_names
+    }
+    try:
+        yield
+    finally:
+        os.environ.update(cluster_env)
+
+
 class TorchDistributedEnvironment:
-    def __init__(self):
+    def __init__(self) -> None:
         """Construct a class holding the parameters required to properly setup
         PyTorch distributed (with the default env:// initialization method).
 
@@ -323,7 +352,11 @@ class TorchDistributedEnvironment:
         # assert MIN_MASTER_PORT <= master_port <= MIN_MASTER_PORT
         return master_port
 
-    def export(self, set_cuda_visible_devices: bool = True) -> "TorchDistributedEnvironment":
+    def export(
+        self,
+        set_cuda_visible_devices: bool = True,
+        overwrite: bool = False,
+    ) -> "TorchDistributedEnvironment":
         """Export all the environment variables required to properly setup
         PyTorch distributed (with the default env:// initialization method) i.e.
         MASTER_ADDR, MASTER_PORT, RANK, WORLD_SIZE (to which LOCAL_RANK and
@@ -334,6 +367,9 @@ class TorchDistributedEnvironment:
         set_cuda_visible_device: bool
             if True, updates CUDA_VISIBLE_DEVICES to use only the device
             matching the local rank.
+        overwrite: bool
+            if True, overwrites the environment variables if they exist;
+            this can be useful when launching a job from another job.
 
         Returns
         --------
@@ -351,9 +387,10 @@ class TorchDistributedEnvironment:
             "LOCAL_RANK": str(self.local_rank),  # Not required
             "LOCAL_WORLD_SIZE": str(self.local_world_size),  # Not required
         }
-        for key in env_vars:
-            if key in os.environ:
-                raise RuntimeError(f"Cannot export environment variables as {key} is already set")
+        if not overwrite:
+            for key in env_vars:
+                if key in os.environ:
+                    raise RuntimeError(f"Cannot export environment variables as {key} is already set")
         # Note: CUDA_VISIBLE_DEVICES may already be set with all available GPUs
         if set_cuda_visible_devices:
             env_vars["CUDA_VISIBLE_DEVICES"] = str(self.local_rank)
